import os, re, maemkit
from glob import glob

class XpcDriver(maemkit.MaemKit):
  options = {}
  mochitestCommand = []
  directories = []

  def __init__(self, mkit):
    self.dirtype = mkit.dirtype
    self.addOptions(mkit.config_options["xpcshell"])

  def addOptions(self, input):
    for option in input: self.options[option] = input[option]


  def readManifest(self, manifest):
    manifestdir = os.path.dirname(manifest)
    try:
      f = open(manifest, "r")
      for line in f:
        dir = line.rstrip()
        path = os.path.join(manifestdir, dir)
        if os.path.isdir(path):
          self.directories.append(path)
      f.close()
    except:
      pass # just eat exceptions

  def getTests(self):
    self.readManifest(self.options["manifest"])

    alltests = []
    for testdir in self.directories:
      testfiles = sorted(glob(os.path.join(testdir, "test_*.js")))
      for testfile in testfiles:
        alltests.append(testfile)

    self.directories = alltests

  def prepTests(self):
    manifest = os.path.normpath(self.options["manifest"])
    self.directories = self.splitListParallel(self.directories, self.options)

  def runTests(self):
    self.mkLogDir(self.options["logdir"])

    for test in self.directories:
      utilname = os.path.normpath(os.path.join(self.options["utility-path"], "runxpcshelltests.py"))
      command = "python " + utilname
      command += " --xre-path=" + os.path.normpath(self.options["xre-path"]) + " " 
      command += " --manifest=" + os.path.normpath(self.options["manifest"]) + " " 
      command += " --test-path=" + test + " " 
      command += os.path.normpath(os.path.join(self.options["xre-path"], "xpcshell")) + " "

      logfile = os.path.join(self.options["logdir"],  self.getLogFileName(test))
      logHandle = open(logfile, "w")
      testResults = self.addCommand(command, timeout=5)
      if (testResults == "*** TIMEOUT ***"):
        testResults = "TEST-UNEXPECTED-FAIL | " + test + " | TIMEOUT "
      logHandle.write(testResults)
      logHandle.close()
      if self.options['verbose'] and os.path.exists(logfile):
        print ''.join(open(logfile).readlines())
    self.cleanup()

  def parseLogs(self):
    self.stitchLogs()
